@@ -17,6 +17,7 @@ from .typing import path_type
 from .util import (
     change_to_path_dir,
     default_config_option_help,
+    get_typehint_origin,
     import_object,
     indent_text,
     is_subclass,
@@ -130,7 +131,7 @@ def _is_action_value_list(action: Action) -> bool:
     return False
 
 
-def _remove_actions(parser, types):
+def remove_actions(parser, types):
 
     def remove(actions):
         rm_actions = [a for a in actions if isinstance(a, types)]
@@ -216,16 +217,24 @@ print_config_skip: ContextVar = ContextVar('print_config_skip', default=False)
 
 
 class _ActionPrintConfig(Action):
-    def __init__(self,
-                 option_strings,
-                 dest=SUPPRESS,
-                 default=SUPPRESS):
-        super().__init__(option_strings=option_strings,
-                         dest=dest,
-                         default=default,
-                         nargs=1,
-                         metavar='[={comments,skip_null,skip_default}+]',
-                         help='Print the configuration after applying all other arguments and exit.')
+    def __init__(
+        self,
+        option_strings,
+        dest=SUPPRESS,
+        default=SUPPRESS,
+    ):
+        super().__init__(
+            option_strings=option_strings,
+            dest=dest,
+            default=default,
+            nargs=1,
+            metavar='\b[=flags]',
+            help=(
+                'Print the configuration after applying all other arguments and exit. The optional '
+                'flags are one or more keywords separated by comma which modify the output. The '
+                'supported flags are: comments, skip_default, skip_null.'
+            ),
+        )
 
     def __call__(self, parser, namespace, value, option_string=None):
         kwargs = {'subparser': parser, 'key': None, 'skip_none': False, 'skip_check': False}
@@ -268,7 +277,7 @@ class _ActionConfigLoad(Action):
 
     def __init__(
         self,
-        basetype: Type = None,
+        basetype: Optional[Type] = None,
         **kwargs
     ):
         if len(kwargs) == 0:
@@ -285,9 +294,13 @@ class _ActionConfigLoad(Action):
             kwargs['_basetype'] = self._basetype
             return _ActionConfigLoad(**kwargs)
         parser, namespace, value = args[:3]
-        cfg_file = Namespace()
-        cfg_file[self.dest] = self._load_config(value, parser)
-        namespace.update(cfg_file)
+        loaded_value = self._load_config(value, parser)
+        if isinstance(namespace.get(self.dest), Namespace):
+            loaded_value = parser.merge_config(
+                Namespace({self.dest: loaded_value}),
+                Namespace({self.dest: namespace[self.dest]})
+            )[self.dest]
+        namespace[self.dest] = loaded_value
 
     def _load_config(self, value, parser):
         try:
@@ -314,7 +327,7 @@ class _ActionHelpClassPath(Action):
 
     def __init__(self, baseclass=None, **kwargs):
         if baseclass is not None:
-            if getattr(baseclass, '__origin__', None) == Union:
+            if get_typehint_origin(baseclass) == Union:
                 baseclasses = [c for c in baseclass.__args__ if c is not NoneType]
                 if len(baseclasses) == 1:
                     baseclass = baseclasses[0]
@@ -325,7 +338,7 @@ class _ActionHelpClassPath(Action):
             super().__init__(**kwargs)
 
     def update_init_kwargs(self, kwargs):
-        if getattr(self._baseclass, '__origin__', None) == Union:
+        if get_typehint_origin(self._baseclass) == Union:
             from .typehints import ActionTypeHint
             self._basename = iter_to_set_str(
                 c.__name__ for c in self._baseclass.__args__
@@ -353,7 +366,7 @@ class _ActionHelpClassPath(Action):
             val_class = import_object(resolve_class_path_by_name(baseclass, value))
         except Exception as ex:
             raise TypeError(f'{option_string}: {ex}')
-        if getattr(self._baseclass, '__origin__', None) == Union:
+        if get_typehint_origin(self._baseclass) == Union:
             baseclasses = self._baseclass.__args__
         else:
             baseclasses = [baseclass]
@@ -362,7 +375,7 @@ class _ActionHelpClassPath(Action):
         dest += '.init_args'
         subparser = type(parser)()
         subparser.add_class_arguments(val_class, dest, **self.sub_add_kwargs)
-        _remove_actions(subparser, (_HelpAction, _ActionPrintConfig, _ActionConfigLoad))
+        remove_actions(subparser, (_HelpAction, _ActionPrintConfig, _ActionConfigLoad))
         args = self.get_args_after_opt(parser.args)
         if args:
             subparser.parse_args(args)
@@ -466,12 +479,12 @@ class ActionParser:
 
     def __init__(
         self,
-        parser: 'ArgumentParser' = None,
+        parser: Optional['ArgumentParser'] = None,
     ):
         """Initializer for ActionParser instance.
 
         Args:
-            parser: A parser to parse the option with.
+            parser (Optional[ArgumentParser]): A parser to parse the option with.
 
         Raises:
             ValueError: If the parser parameter is invalid.
@@ -538,8 +551,8 @@ class ActionParser:
 
 
 single_subcommand: ContextVar = ContextVar('single_subcommand', default=True)
-
 parent_parsers: ContextVar = ContextVar('parent_parsers', default=[])
+parse_kwargs: ContextVar = ContextVar('parse_kwargs', default={})
 
 
 @contextmanager
@@ -579,6 +592,7 @@ class _ActionSubCommands(_SubParsersAction):
         parser.default_env = self.parent_parser.default_env
         parser.parent_parser = self.parent_parser
         parser.parser_mode = self.parent_parser.parser_mode
+        parser.error_handler = self.parent_parser.error_handler
         parser.subcommand = name
 
         # create a pseudo-action to hold the choice help
@@ -608,10 +622,15 @@ class _ActionSubCommands(_SubParsersAction):
         if subcommand in self._name_parser_map:
             subparser = self._name_parser_map[subcommand]
             subnamespace = namespace.get(subcommand).clone() if subcommand in namespace else None
-            subnamespace, unk = subparser.parse_known_args(arg_strings, namespace=subnamespace)
-            if unk:
-                raise ParserError(f'Unrecognized arguments: {" ".join(unk)}')
-            namespace.update(subnamespace, subcommand)
+            kwargs = dict(_skip_check=True, **parse_kwargs.get())
+            namespace[subcommand] = subparser.parse_args(arg_strings, namespace=subnamespace, **kwargs)
+
+
+    @staticmethod
+    @contextmanager
+    def parse_kwargs_context(kwargs):
+        parse_kwargs.set(kwargs)
+        yield
 
 
     @staticmethod
@@ -727,7 +746,7 @@ class ActionPathList(Action, FilesCompleterMethod):
 
     def __init__(
         self,
-        mode: str = None,
+        mode: Optional[str] = None,
         rel: str = 'cwd',
         **kwargs
     ):
