@@ -7,7 +7,6 @@ import logging
 import os
 import re
 import sys
-from copy import deepcopy
 from typing import Any, Callable, Dict, List, NoReturn, Optional, Sequence, Set, Tuple, Type, Union
 
 from .formatters import DefaultHelpFormatter, empty_help, formatter_context, get_env_var
@@ -24,7 +23,8 @@ from .loaders_dumpers import (
     set_omegaconf_loader,
     yaml_load,
 )
-from .namespace import is_meta_key, Namespace, patch_namespace, split_key, split_key_leaf, strip_meta
+from .namespace import is_meta_key, Namespace, patch_namespace, recreate_branches, split_key, split_key_leaf, strip_meta
+from .parameter_resolvers import ConditionalDefault
 from .signatures import is_pure_dataclass, SignatureArguments
 from .typehints import ActionTypeHint, is_subclass_spec
 from .typing import is_final_class
@@ -91,7 +91,7 @@ class ActionsContainer(SignatureArguments, argparse._ActionsContainer):
         Args:
             enable_path: Whether to try parsing path/subconfig when argument is a complex type.
         """
-        parser = self.parser if hasattr(self, 'parser') else self  # type: ignore
+        parser = self.parser if hasattr(self, 'parser') else self
         if 'action' in kwargs:
             if isinstance(kwargs['action'], ActionParser):
                 if kwargs['action']._parser == parser:
@@ -131,7 +131,7 @@ class ActionsContainer(SignatureArguments, argparse._ActionsContainer):
         return action
 
 
-    def add_argument_group(self, *args, name: str = None, **kwargs):
+    def add_argument_group(self, *args, name: Optional[str] = None, **kwargs):
         """Adds a group to the parser.
 
         All the arguments from `argparse.ArgumentParser.add_argument_group
@@ -147,7 +147,7 @@ class ActionsContainer(SignatureArguments, argparse._ActionsContainer):
         Raises:
             ValueError: If group with the same name already exists.
         """
-        parser = self.parser if hasattr(self, 'parser') else self  # type: ignore
+        parser = self.parser if hasattr(self, 'parser') else self
         if name is not None and name in parser.groups:
             raise ValueError(f'Group with name {name} already exists.')
         group = _ArgumentGroup(parser, *args, logger=parser.logger, **kwargs)
@@ -167,7 +167,7 @@ class _ArgumentGroup(ActionsContainer, argparse._ArgumentGroup):
 class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser):
     """Parser for command line, yaml/jsonnet files and environment variables."""
 
-    formatter_class: Type[DefaultHelpFormatter]  # type: ignore
+    formatter_class: Type[DefaultHelpFormatter]
     groups: Optional[Dict[str, '_ArgumentGroup']] = None
     _subcommands_action: Optional[_ActionSubCommands] = None
 
@@ -212,7 +212,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
             self.groups = {}
         self.required_args: Set[str] = set()
         self.save_path_content: Set[str] = set()
-        self.default_config_files = default_config_files
+        self.default_config_files = default_config_files  # type: ignore
         self.default_meta = default_meta
         self.default_env = default_env
         self.env_prefix = env_prefix
@@ -241,9 +241,6 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
                 self.error(f'All arguments are expected to be strings: {args}')
         self.args = args
 
-        if namespace is None:
-            namespace = Namespace()
-
         namespace = argcomplete_namespace(caller, self, namespace)
 
         try:
@@ -302,9 +299,6 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
             with lenient_check_context():
                 ActionTypeHint.add_sub_defaults(self, cfg)
 
-        if not (with_meta or (with_meta is None and self._default_meta)):
-            cfg = strip_meta(cfg)
-
         _ActionPrintConfig.print_config_if_requested(self, cfg)
 
         with load_value_context(self.parser_mode):
@@ -313,6 +307,9 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
             if not skip_check and not lenient_check.get():
                 self.check_config(cfg, skip_required=skip_required)
 
+        if not (with_meta or (with_meta is None and self._default_meta)):
+            cfg = strip_meta(cfg)
+
         if log_message is not None:
             self._logger.info(log_message)
 
@@ -323,7 +320,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         self,
         defaults: bool = True,
         env: Optional[bool] = None,
-        environ: Union[Dict[str, str], os._Environ] = None,
+        environ: Optional[Union[Dict[str, str], os._Environ]] = None,
     ):
         cfg = Namespace()
         if defaults:
@@ -342,7 +339,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
     def parse_args(  # type: ignore[override]
         self,
         args: Optional[Sequence[str]] = None,
-        namespace: Namespace = None,
+        namespace: Optional[Namespace] = None,
         env: Optional[bool] = None,
         defaults: bool = True,
         with_meta: Optional[bool] = None,
@@ -375,7 +372,8 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
             if namespace:
                 cfg = self.merge_config(namespace, cfg)
 
-            cfg, unk = self.parse_known_args(args=args, namespace=cfg)
+            with _ActionSubCommands.parse_kwargs_context({'env': env, 'defaults': defaults}):
+                cfg, unk = self.parse_known_args(args=args, namespace=cfg)
             if unk:
                 self.error(f'Unrecognized arguments: {" ".join(unk)}')
 
@@ -478,7 +476,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
 
     def parse_env(
         self,
-        env: Dict[str, str] = None,
+        env: Optional[Dict[str, str]] = None,
         defaults: bool = True,
         with_meta: Optional[bool] = None,
         **kwargs,
@@ -631,7 +629,9 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         try:
             cfg_dict = load_value(cfg_str, path=cfg_path, ext_vars=ext_vars)
         except get_loader_exceptions() as ex:
-            raise TypeError(f'Problems parsing config :: {ex}') from ex
+            raise TypeError(f'Problems parsing config: {ex}') from ex
+        if not isinstance(cfg_dict, dict):
+            raise TypeError(f'Unexpected config: {cfg_str}')
         return self._apply_actions(cfg_dict, prev_cfg=prev_cfg)
 
 
@@ -699,7 +699,6 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         """
         check_valid_dump_format(format)
 
-        cfg = deepcopy(cfg)
         cfg = strip_meta(cfg)
         ActionLink.strip_link_target_keys(self, cfg)
 
@@ -771,7 +770,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         skip_check: bool = False,
         overwrite: bool = False,
         multifile: bool = True,
-        branch: str = None,
+        branch: Optional[str] = None,
     ) -> None:
         """Writes to file(s) the yaml or json for the given configuration object.
 
@@ -817,7 +816,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
                 f.write(self.dump(cfg, **dump_kwargs))  # type: ignore
 
         else:
-            cfg = deepcopy(cfg)
+            cfg = cfg.clone()
             ActionLink.strip_link_target_keys(self, cfg)
 
             if not skip_check:
@@ -859,11 +858,11 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
 
     ## Methods related to defaults ##
 
-    def set_defaults(self, *args, **kwargs) -> None:
+    def set_defaults(self, *args: Dict[str, Any], **kwargs: Any) -> None:
         """Sets default values from dictionary or keyword arguments.
 
         Args:
-            *args (dict): Dictionary defining the default values to set.
+            *args: Dictionary defining the default values to set.
             **kwargs: Sets default values based on keyword arguments.
 
         Raises:
@@ -936,7 +935,6 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         """Returns a namespace with all default values.
 
         Args:
-            nested: Whether the namespace should be nested.
             skip_check: Whether to skip check if configuration is valid.
 
         Returns:
@@ -944,8 +942,12 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         """
         cfg = Namespace()
         for action in filter_default_actions(self._actions):
-            if action.default != argparse.SUPPRESS and action.dest != argparse.SUPPRESS:
-                cfg[action.dest] = deepcopy(action.default)
+            if (
+                action.default != argparse.SUPPRESS and
+                action.dest != argparse.SUPPRESS and
+                not isinstance(action.default, ConditionalDefault)
+            ):
+                cfg[action.dest] = recreate_branches(action.default)
 
         self._logger.info('Loaded default values from parser.')
 
@@ -984,7 +986,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
 
     ## Other methods ##
 
-    def error(self, message: str, ex: Exception = None) -> NoReturn:
+    def error(self, message: str, ex: Optional[Exception] = None) -> NoReturn:
         """Logs error message if a logger is set, calls the error handler and raises a ParserError."""
         self._logger.error(message)
         if self._error_handler is not None:
@@ -1000,7 +1002,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         cfg: Namespace,
         skip_none: bool = True,
         skip_required: bool = False,
-        branch: str = None,
+        branch: Optional[str] = None,
     ) -> None:
         """Checks that the content of a given configuration object conforms with the parser.
 
@@ -1136,7 +1138,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
         Returns:
             The stripped configuration object.
         """
-        cfg = deepcopy(cfg)
+        cfg = cfg.clone()
 
         del_keys = []
         for key in cfg.keys():
@@ -1292,7 +1294,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
             if isinstance(value, str):
                 value = action.check_type(value, self)
         elif hasattr(action, '_check_type'):
-            value = action._check_type(value, cfg=cfg)  # type: ignore
+            value = action._check_type(value, cfg=cfg)
         elif action.type is not None:
             try:
                 if action.nargs in {None, '?'} or action.nargs == 0:
@@ -1308,7 +1310,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
     ## Properties ##
 
     @property
-    def error_handler(self):
+    def error_handler(self) -> Optional[Callable[['ArgumentParser', str], None]]:
         """Property for the error_handler function that is called when there are parsing errors.
 
         :getter: Returns the current error_handler function.
@@ -1321,7 +1323,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
 
 
     @error_handler.setter
-    def error_handler(self, error_handler):
+    def error_handler(self, error_handler: Optional[Callable[['ArgumentParser', str], None]]):
         if callable(error_handler) or error_handler is None:
             self._error_handler = error_handler
         else:
@@ -1329,7 +1331,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
 
 
     @property
-    def default_config_files(self):
+    def default_config_files(self) -> List[str]:
         """Default config file locations.
 
         :getter: Returns the current default config file locations.
@@ -1342,7 +1344,7 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
 
 
     @default_config_files.setter
-    def default_config_files(self, default_config_files:Optional[List[str]]):
+    def default_config_files(self, default_config_files: Optional[List[str]]):
         if default_config_files is None:
             self._default_config_files = []
         elif isinstance(default_config_files, list) and all(isinstance(x, str) for x in default_config_files):
@@ -1385,6 +1387,9 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
             self._default_env = default_env
         else:
             raise ValueError('default_env has to be a boolean.')
+        if self._subcommands_action:
+            for subparser in self._subcommands_action._name_parser_map.values():
+                subparser.default_env = self._default_env
 
 
     @property
@@ -1485,5 +1490,5 @@ class ArgumentParser(ActionsContainer, ArgumentLinking, argparse.ArgumentParser)
 
 from .deprecated import parse_as_dict_patch, instantiate_subclasses_patch
 instantiate_subclasses_patch()
-if 'JSONARGPARSE_SKIP_DEPRECATION_PATCH' not in os.environ:
+if 'SPHINX_BUILD' not in os.environ:
     parse_as_dict_patch()
