@@ -6,8 +6,8 @@ import os
 import typing
 from contextlib import contextmanager
 from importlib.util import find_spec
+from typing import Optional
 
-from .namespace import Namespace
 
 __all__ = [
     'get_config_read_mode',
@@ -41,10 +41,22 @@ def typing_extensions_import(name):
         return getattr(typing, name, False)
 
 
+def is_compatible_final(final) -> bool:
+    @final
+    class FinalClass:
+        pass
+    return getattr(FinalClass, '__final__', False)
+
+
 final = typing_extensions_import('final')
-if not final or not getattr(final(Namespace()), '__final__', False):
+if not final or not is_compatible_final(final) or 'SPHINX_BUILD' in os.environ:
     def final(cls):  # pylint: disable=function-redefined
-        """Decorator to make a class ``final``, i.e. it shouldn't be subclassed."""
+        """Decorator to make a class ``final``, i.e., it shouldn't be subclassed.
+
+        It is the same as ``typing.final`` or an equivalent implementation
+        depending on the python version and whether typing-extensions is
+        installed.
+        """
         setattr(cls, '__final__', True)
         return cls
 
@@ -155,7 +167,7 @@ def get_config_read_mode() -> str:
     return _config_read_mode
 
 
-def set_docstring_parse_options(style = None, attribute_docstrings: bool = None):
+def set_docstring_parse_options(style = None, attribute_docstrings: Optional[bool] = None):
     """Sets options for docstring parsing.
 
     Args:
@@ -243,7 +255,7 @@ def argcomplete_autocomplete(parser):
 
 def argcomplete_namespace(caller, parser, namespace):
     if caller == 'argcomplete':
-        namespace.__class__ = Namespace
+        namespace.__class__ = __import__('jsonargparse').Namespace
         namespace = parser.merge_config(parser.get_defaults(skip_check=True), namespace).as_flat()
     return namespace
 
