@@ -10,8 +10,16 @@ from dataclasses import dataclass
 from functools import partial
 from typing import Any, Callable, List, Optional, Tuple, Type, Union
 
-from .util import ClassFromFunctionBase, is_subclass, LoggerProperty, parse_logger
 from .optionals import parse_docs
+from .util import (
+    ClassFromFunctionBase,
+    get_import_path,
+    is_subclass,
+    iter_to_set_str,
+    LoggerProperty,
+    parse_logger,
+    unique,
+)
 
 
 @dataclass
@@ -22,19 +30,39 @@ class ParamData:
     kind: Optional[inspect._ParameterKind] = None
     doc: Optional[str] = None
     component: Optional[Union[Callable, Type, Tuple]] = None
-    parent: Optional[Type] = None
+    parent: Optional[Union[Type, Tuple]] = None
+    origin: Optional[Union[str, Tuple]] = None
 
 
 ParamList = List[ParamData]
 parameter_attributes = [s[1:] for s in inspect.Parameter.__slots__]  # type: ignore
 kinds = inspect._ParameterKind
 ast_assign_type: Tuple[Type[ast.AST], ...] = (ast.AnnAssign, ast.Assign)
+param_kwargs_get = '**.get()'
+param_kwargs_pop = '**.pop()'
 
 
 class SourceNotAvailable(Exception):
     "Raised when the source code for some component is not available."
 
 
+class ConditionalDefault(list):
+    def __repr__(self):
+        return iter_to_set_str(self, sep=', ')
+
+
+def get_parameter_origins(component, parent) -> str:
+    if isinstance(component, tuple):
+        assert parent is None or len(component) == len(parent)
+        return iter_to_set_str(
+            get_parameter_origins(c, parent[n] if parent else None)
+            for n, c in enumerate(component)
+        )
+    if parent:
+        return f'{get_import_path(parent)}.{component.__name__}'
+    return get_import_path(component)
+
+
 def is_staticmethod(attr) -> bool:
     return isinstance(attr, staticmethod)
 
@@ -55,6 +83,10 @@ def is_classmethod(parent, component) -> bool:
     return parent and isinstance(inspect.getattr_static(parent, component.__name__), classmethod)
 
 
+def is_lambda(value: Any) -> bool:
+    return callable(value) and value.__name__ == '<lambda>'
+
+
 def ast_str(node):
     return getattr(ast, 'unparse', ast.dump)(node)
 
@@ -75,6 +107,10 @@ def ast_get_assign_targets(node):
     return node.targets if isinstance(node, ast.Assign) else [node.target]
 
 
+def ast_is_not(node) -> bool:
+    return isinstance(node, ast.UnaryOp) and isinstance(node.op, ast.Not)
+
+
 dict_ast = ast.dump(ast_variable_load('dict'))
 
 
@@ -117,6 +153,16 @@ def ast_get_constant_value(node):
     return getattr(node, ast_constant_attr[node.__class__])
 
 
+def ast_get_name_and_attrs(node) -> List[str]:
+    names = []
+    while isinstance(node, ast.Attribute):
+        names.append(node.attr)
+        node = node.value
+    if isinstance(node, ast.Name):
+        names.append(node.id)
+    return names[::-1]
+
+
 def ast_is_kwargs_pop_or_get(node, value_dump) -> bool:
     return (
         isinstance(node.func, ast.Attribute) and
@@ -137,7 +183,7 @@ def ast_is_super_call(node) -> bool:
     )
 
 
-def ast_is_supported_super_call(node, self_name, logger) -> bool:
+def ast_is_supported_super_call(node, self_name, log_debug) -> bool:
     supported = False
     args = node.func.value.args
     if not args and not node.func.value.keywords:
@@ -157,7 +203,7 @@ def ast_is_supported_super_call(node, self_name, logger) -> bool:
                 supported = True
                 break
     if not supported:
-        logger.debug(f'AST resolver: unsupported super parameters: {ast_str(node)}')
+        log_debug(f'unsupported super parameters: {ast_str(node)}')
     return supported
 
 
@@ -224,7 +270,7 @@ ast_literals = {
 }
 
 
-def get_kwargs_pop_or_get_parameter(node, component, parent, doc_params, logger):
+def get_kwargs_pop_or_get_parameter(node, component, parent, doc_params, log_debug):
     name = ast_get_constant_value(node.args[0])
     if ast_is_constant(node.args[1]):
         default = ast_get_constant_value(node.args[1])
@@ -234,15 +280,30 @@ def get_kwargs_pop_or_get_parameter(node, component, parent, doc_params, logger)
             default = ast_literals[default]()
         else:
             default = None
-            logger.debug(f'AST resolver: unsupported kwargs pop/get default: {ast_str(node)}')
+            log_debug(f'unsupported kwargs pop/get default: {ast_str(node)}')
     return ParamData(
         name=name,
         annotation=inspect._empty,
         default=default,
-        kind=inspect._ParameterKind.KEYWORD_ONLY,
+        kind=kinds.KEYWORD_ONLY,
         doc=doc_params.get(name),
         parent=parent,
         component=component,
+        origin=param_kwargs_get if node.func.attr == 'get' else param_kwargs_pop,
+    )
+
+
+def is_param_subclass_instance_default(param: ParamData) -> bool:
+    from .typehints import ActionTypeHint, get_subclass_types
+    class_types = get_subclass_types(param.annotation)
+    return (
+        (class_types and isinstance(param.default, class_types)) or
+        (
+            is_lambda(param.default) and
+            ActionTypeHint.is_callable_typehint(param.annotation, all_subtypes=False) and
+            param.annotation.__args__ and
+            ActionTypeHint.is_subclass_typehint(param.annotation.__args__[-1], all_subtypes=False)
+        )
     )
 
 
@@ -266,35 +327,41 @@ def replace_args_and_kwargs(params: ParamList, args: ParamList, kwargs: ParamLis
     return params
 
 
-def common_parameters(params_list: List[ParamList]) -> ParamList:
+def group_parameters(params_list: List[ParamList]) -> ParamList:
     if len(params_list) == 1:
+        for param in params_list[0]:
+            if not isinstance(param.origin, tuple):
+                param.origin = None
         return params_list[0]
-    common = []
+    grouped = []
+    params_count = 0
+    params_skip = set()
     params_dict = defaultdict(lambda: [])
     for params in params_list:
+        if params[0].origin not in {param_kwargs_get, param_kwargs_pop}:
+            params_count += 1
         for param in params:
-            params_dict[param.name].append(param)
+            if param.name not in params_skip and param.kind != kinds.POSITIONAL_ONLY:
+                params_dict[param.name].append(param)
+                if param.origin == param_kwargs_pop:
+                    params_skip.add(param.name)
     for params in params_dict.values():
-        if (
-            len(params) == len(params_list) and
-            len(set(p.annotation for p in params)) == 1 and
-            kinds.POSITIONAL_ONLY not in set(p.kind for p in params)
-        ):
-            common.append(params[0])
-    return common
-
-
-def merge_parameters(source: Union[ParamData, ParamList], target: ParamList) -> ParamList:
-    if not isinstance(source, list):
-        source = [source]
-    source_dict = {p.name: p for p in source}
-    replace_names = set()
-    for p in target:
-        if p.name in source_dict and p.annotation is inspect._empty and source_dict[p.name] is not inspect._empty:
-            replace_names.add(p.name)
-    target = [p for p in target if p.name not in replace_names]
-    target_names = set(p.name for p in target)
-    return target + [s for s in source if s.name not in target_names]
+        gparam = params[0]
+        types = unique(p.annotation for p in params if p.annotation is not inspect._empty)
+        defaults = unique(p.default for p in params if p.default is not inspect._empty)
+        if len(params) >= params_count and len(types) <= 1 and len(defaults) <= 1:
+            gparam.origin = None
+        else:
+            gparam.parent = tuple(p.parent for p in params)
+            gparam.component = tuple(p.component for p in params)
+            gparam.origin = tuple(p.origin for p in params)
+            gparam.default = ConditionalDefault((p.default for p in params) if len(defaults) > 1 else defaults)
+            if len(types) > 1:
+                gparam.annotation = Union[tuple(types)] if types else inspect._empty
+        docs = [p.doc for p in params if p.doc]
+        gparam.doc = docs[0] if docs else None
+        grouped.append(gparam)
+    return grouped
 
 
 def has_dunder_new_method(cls, attr_name):
@@ -380,11 +447,13 @@ class ParametersVisitor(LoggerProperty, ast.NodeVisitor):
     ):
         super().__init__(**kwargs)
         self.component, self.parent, _ = get_component_and_parent(function_or_class, method_or_property)
-        self.parse_source_tree()
+
+    def log_debug(self, message) -> None:
+        self.logger.debug(f'AST resolver: {message}')
 
     def parse_source_tree(self):
-        """Parses the component's module AST and sets the component and parent nodes."""
-        if self.component is None:
+        """Parses the component's AST and sets the component and parent nodes."""
+        if hasattr(self, 'component_node'):
             return
         try:
             source = textwrap.dedent(inspect.getsource(self.component))
@@ -431,6 +500,18 @@ class ParametersVisitor(LoggerProperty, ast.NodeVisitor):
                 self.values_found.append((key, node))
         self.generic_visit(node)
 
+    def visit_If(self, node):
+        is_test_not = ast_is_not(node.test)
+        test_node = node.test.operand if is_test_not else node.test
+        component_globals = self.component.__globals__
+        if isinstance(test_node, ast.Name) and test_node.id in component_globals:
+            condition = bool(component_globals[test_node.id])
+            if is_test_not:
+                condition = not condition
+            body = node.body if condition else node.orelse
+            node = ast.If(test=ast.Constant(value=True), body=body, orelse=[])
+        self.generic_visit(node)
+
     def find_values_usage(self, values):
         self.find_values = values
         self.values_found = []
@@ -444,21 +525,21 @@ class ParametersVisitor(LoggerProperty, ast.NodeVisitor):
         if isinstance(node.func, ast.Name):
             if is_classmethod(self.parent, self.component) and node.func.id == self.self_name:
                 function_or_class = self.parent
-            else:
+            elif hasattr(module, node.func.id):
                 function_or_class = getattr(module, node.func.id)
         elif isinstance(node.func, ast.Attribute) and isinstance(node.func.value, ast.Name):
             if self.parent and ast.dump(node.func.value) == ast.dump(ast_variable_load(self.self_name)):
                 function_or_class = self.parent
                 method_or_property = node.func.attr
-            else:
+            elif hasattr(module, node.func.value.id):
                 container = getattr(module, node.func.value.id)
                 if inspect.isclass(container):
                     function_or_class = container
                     method_or_property = node.func.attr
-                else:
+                elif hasattr(container, node.func.attr):
                     function_or_class = getattr(container, node.func.attr)
         if not function_or_class:
-            self.logger.debug(f'AST resolver: not supported: {ast_str(node)}')
+            self.log_debug(f'not supported: {ast_str(node)}')
             return None
         return function_or_class, method_or_property
 
@@ -470,7 +551,7 @@ class ParametersVisitor(LoggerProperty, ast.NodeVisitor):
             kwarg = ast_get_call_kwarg_with_value(node, value)
             if kwarg:
                 if kwarg.arg:
-                    self.logger.debug(f'AST resolver: kwargs attribute given as keyword parameter not supported: {ast_str(node)}')
+                    self.log_debug(f'kwargs attribute given as keyword parameter not supported: {ast_str(node)}')
                 else:
                     get_param_args = self.get_node_component(node)
                     if get_param_args:
@@ -478,7 +559,54 @@ class ParametersVisitor(LoggerProperty, ast.NodeVisitor):
             params = remove_given_parameters(node, params)
         return params
 
+    def replace_param_default_subclass_specs(self, params: List[ParamData]) -> None:
+        params = [p for p in params if is_param_subclass_instance_default(p)]
+        if params:
+            self.parse_source_tree()
+            default_nodes = self.get_default_nodes({p.name for p in params})
+            assert len(params) == len(default_nodes)
+            from .typehints import get_subclass_types
+            for param, default_node in zip(params, default_nodes):
+                lambda_default = is_lambda(param.default)
+                node = default_node
+                num_positionals = 0
+                if lambda_default:
+                    node = default_node.body
+                    num_positionals = len(param.annotation.__args__) - 1
+                class_type = self.get_call_class_type(node)
+                subclass_types = get_subclass_types(param.annotation)
+                if not (class_type and subclass_types and is_subclass(class_type, subclass_types)):
+                    continue
+                subclass_spec = dict(class_path=get_import_path(class_type), init_args=dict())
+                for kwarg in node.keywords:
+                    if kwarg.arg and ast_is_constant(kwarg.value):
+                        subclass_spec['init_args'][kwarg.arg] = ast_get_constant_value(kwarg.value)
+                    else:
+                        subclass_spec.clear()
+                        break
+                if not subclass_spec or len(node.args) - num_positionals > 0:
+                    self.log_debug(f'unsupported class instance default: {ast_str(default_node)}')
+                elif subclass_spec:
+                    if not subclass_spec['init_args']:
+                        del subclass_spec['init_args']
+                    param.default = subclass_spec
+
+    def get_call_class_type(self, node) -> Optional[type]:
+        names = ast_get_name_and_attrs(getattr(node, 'func', None))
+        class_type = self.component.__globals__.get(names[0]) if names else None
+        for name in names[1:]:
+            class_type = getattr(class_type, name, None)
+        return class_type if inspect.isclass(class_type) else None
+
+    def get_default_nodes(self, param_names: set):
+        node = self.component_node.args
+        arg_nodes = getattr(node, 'posonlyargs', []) + node.args
+        default_nodes = [None] * (len(arg_nodes) - len(node.defaults)) + node.defaults
+        default_nodes = [d for n, d in enumerate(default_nodes) if arg_nodes[n].arg in param_names]
+        return default_nodes
+
     def get_parameters_args_and_kwargs(self) -> Tuple[ParamList, ParamList]:
+        self.parse_source_tree()
         args_name = getattr(self.component_node.args.vararg, 'arg', None)
         kwargs_name = getattr(self.component_node.args.kwarg, 'arg', None)
         values_to_find = {}
@@ -487,43 +615,45 @@ class ParametersVisitor(LoggerProperty, ast.NodeVisitor):
         if kwargs_name:
             values_to_find[kwargs_name] = kwargs_value = ast_variable_load(kwargs_name)
 
-        args: ParamList = []
-        kwargs: ParamList = []
         values_found = self.find_values_usage(values_to_find)
         if not values_found:
-            return args, kwargs
+            return [], []
 
+        params_list = []
         kwargs_value_dump = ast.dump(kwargs_value)
         for node in [v for k, v in values_found if k == kwargs_name]:
             if isinstance(node, ast.Call):
                 if ast_is_kwargs_pop_or_get(node, kwargs_value_dump):
-                    param = get_kwargs_pop_or_get_parameter(node, self.component, self.parent, self.doc_params, self.logger)
-                    kwargs = merge_parameters(param, kwargs)
+                    param = get_kwargs_pop_or_get_parameter(node, self.component, self.parent, self.doc_params, self.log_debug)
+                    params_list.append([param])
                     continue
                 kwarg = ast_get_call_kwarg_with_value(node, kwargs_value)
                 params = []
                 if kwarg.arg:
-                    self.logger.debug(f'AST resolver: kwargs given as keyword parameter not supported: {ast_str(node)}')
+                    self.log_debug(f'kwargs given as keyword parameter not supported: {ast_str(node)}')
                 elif self.parent and ast_is_super_call(node):
-                    if ast_is_supported_super_call(node, self.self_name, self.logger):
+                    if ast_is_supported_super_call(node, self.self_name, self.log_debug):
                         params = get_mro_parameters(node.func.attr, get_signature_parameters, self.logger)  # type: ignore
                 else:
                     get_param_args = self.get_node_component(node)
                     if get_param_args:
                         params = get_signature_parameters(*get_param_args, logger=self.logger)
-                args, kwargs_ = split_args_and_kwargs(remove_given_parameters(node, params))
-                kwargs = merge_parameters(kwargs_, kwargs)
-                break
+                params = remove_given_parameters(node, params)
+                if params:
+                    self.add_node_origins(params, node)
+                    params_list.append(params)
             elif isinstance(node, ast_assign_type):
                 self_attr = self.parent and ast_is_attr_assign(node, self.self_name)
                 if self_attr:
                     params = self.get_parameters_attr_use_in_members(self_attr)
-                    kwargs = merge_parameters(params, kwargs)
-                    break
+                    if params:
+                        self.add_node_origins(params, node)
+                        params_list.append(params)
                 else:
-                    self.logger.debug(f'AST resolver: unsupported type of assign: {ast_str(node)}')
+                    self.log_debug(f'unsupported type of assign: {ast_str(node)}')
 
-        return args, kwargs
+        params = group_parameters(params_list)
+        return split_args_and_kwargs(params)
 
     def get_parameters_attr_use_in_members(self, attr_name) -> ParamList:
         attr_value = ast_attribute_load(self.self_name, attr_name)
@@ -537,25 +667,36 @@ class ParametersVisitor(LoggerProperty, ast.NodeVisitor):
             kwargs = visitor.get_parameters_call_attr(attr_name, attr_value)
             if kwargs is not None:
                 return kwargs
-        self.logger.debug(f'AST resolver: did not find use of {self.self_name}.{attr_name} in members of {self.parent}')
+        self.log_debug(f'did not find use of {self.self_name}.{attr_name} in members of {self.parent}')
         return []
 
+    def add_node_origins(self, params: ParamList, node) -> None:
+        origin = None
+        for param in params:
+            if param.origin is None:
+                if not origin:
+                    origin = f'{get_parameter_origins(self.component, self.parent)}:{node.lineno}'
+                param.origin = origin
+
     def get_parameters_call_attr(self, attr_name: str, attr_value: ast.AST) -> Optional[ParamList]:
+        self.parse_source_tree()
         values_to_find = {attr_name: attr_value}
         values_found = self.find_values_usage(values_to_find)
         matched = []
         if values_found:
             for _, node in values_found:
                 match = self.match_call_that_uses_attr(node, attr_name)
-                if match is not None:
+                if match:
+                    self.add_node_origins(match, node)
                     matched.append(match)
-            matched = common_parameters(matched)
+            matched = group_parameters(matched)
         return matched or None
 
     def get_parameters(self) -> ParamList:
         if self.component is None:
             return []
         params, args_idx, kwargs_idx, doc_params = get_signature_parameters_and_indexes(self.component, self.parent, self.logger)
+        self.replace_param_default_subclass_specs(params)
         if args_idx >= 0 or kwargs_idx >= 0:
             self.doc_params = doc_params
             with mro_context(self.parent):
@@ -606,9 +747,14 @@ def get_signature_parameters(
         visitor = ParametersVisitor(function_or_class, method_or_property, logger=logger)
         return visitor.get_parameters()
     except Exception as ex:
-        cause = 'Source not available' if isinstance(ex, SourceNotAvailable) else 'Problems with AST resolving'
+        cause = 'Source not available'
+        exc_info = None
+        if not isinstance(ex, SourceNotAvailable):
+            cause = 'Problems with AST resolving'
+            exc_info = ex
         logger.debug(
             f'{cause}, falling back to parameters by assumptions: function_or_class={function_or_class} '
-            f'method_or_property={method_or_property} :: {ex}'
+            f'method_or_property={method_or_property}: {ex}',
+            exc_info=exc_info,
         )
         return get_parameters_by_assumptions(function_or_class, method_or_property, logger)
