@@ -23,6 +23,8 @@ from typing import (
     MutableMapping,
     MutableSequence,
     MutableSet,
+    NoReturn,
+    Optional,
     Sequence,
     Set,
     Tuple,
@@ -44,6 +46,7 @@ from .util import (
     change_to_path_dir,
     ClassType,
     get_import_path,
+    get_typehint_origin,
     import_object,
     indent_text,
     is_subclass,
@@ -107,7 +110,7 @@ class ActionTypeHint(Action):
 
     def __init__(
         self,
-        typehint: Type = None,
+        typehint: Optional[Type] = None,
         enable_path: bool = False,
         **kwargs
     ):
@@ -281,12 +284,17 @@ class ActionTypeHint(Action):
 
 
     @staticmethod
-    def discard_init_args_on_class_path_change(parser, cfg_to, cfg_from):
-        for action in [a for a in parser._actions if ActionTypeHint.is_subclass_typehint(a, all_subtypes=False)]:
-            val_to = cfg_to.get(action.dest)
-            val_from = cfg_from.get(action.dest)
-            if is_subclass_spec(val_to) and is_subclass_spec(val_from):
-                discard_init_args_on_class_path_change(action, val_to, val_from)
+    def discard_init_args_on_class_path_change(parser_or_action, prev_cfg, cfg):
+        keys = list(prev_cfg.keys(branches=True))
+        for key in keys:
+            prev_val = prev_cfg.get(key)
+            val = cfg.get(key)
+            if is_subclass_spec(prev_val) and is_subclass_spec(val):
+                action = parser_or_action
+                if not isinstance(parser_or_action, ActionTypeHint):
+                    action = _find_action(parser_or_action, key)
+                if isinstance(action, ActionTypeHint):
+                    discard_init_args_on_class_path_change(action, prev_val, val)
 
 
     @staticmethod
@@ -355,16 +363,22 @@ class ActionTypeHint(Action):
             if 'nargs' in kwargs and kwargs['nargs'] == 0:
                 raise ValueError('ActionTypeHint does not allow nargs=0.')
             return ActionTypeHint(**kwargs)
-        if self.nargs == '?' and args[2] is None:
-            val = None
-        else:
-            cfg, val, opt_str = args[1:]
+        cfg, val, opt_str = args[1:]
+        if not (self.nargs == '?' and val is None):
             if isinstance(opt_str, str) and opt_str.startswith(f'--{self.dest}.'):
                 sub_opt = opt_str[len(f'--{self.dest}.'):]
                 val = NestedArg(key=sub_opt, val=val)
             append = opt_str == f'--{self.dest}+'
             val = self._check_type(val, append=append, cfg=cfg)
-        args[1].update(val, self.dest)
+            if is_subclass_spec(val):
+                prev_val = cfg.get(self.dest)
+                if is_subclass_spec(prev_val) and 'init_args' in prev_val:
+                    ActionTypeHint.discard_init_args_on_class_path_change(
+                        self,
+                        prev_val.init_args,
+                        val.get('init_args'),
+                    )
+        cfg.update(val, self.dest)
 
 
     def _check_type(self, value, append=False, cfg=None):
@@ -382,7 +396,7 @@ class ActionTypeHint(Action):
                 path_meta = val.pop('__path__', None) if isinstance(val, dict) else None
 
                 prev_val = cfg.get(self.dest) if cfg else None
-                if not prev_val and not sub_defaults.get() and is_subclass_spec(self.default):
+                if prev_val is None and not sub_defaults.get() and is_subclass_spec(self.default):
                     prev_val = Namespace(class_path=self.default.class_path)
 
                 kwargs = {
@@ -401,7 +415,7 @@ class ActionTypeHint(Action):
                             ex = None
                     except ValueError:
                         if self._enable_path and config_path is None and isinstance(orig_val, str):
-                            msg = f'\n- Expected a config path but "{orig_val}" either not accessible or invalid.\n- '
+                            msg = f'\n- Expected a config path but {orig_val} either not accessible or invalid\n- '
                             raise type(ex)(msg+str(ex)) from ex
                     if ex:
                         raise ex
@@ -418,7 +432,8 @@ class ActionTypeHint(Action):
                 value[num] = val
             except (TypeError, ValueError) as ex:
                 elem = '' if not islist else f' element {num+1}'
-                raise TypeError(f'Parser key "{self.dest}"{elem}: {ex}') from ex
+                error = indent_text(str(ex))
+                raise TypeError(f'Parser key "{self.dest}"{elem}: {error}') from ex
         return value if islist else value[0]
 
 
@@ -484,6 +499,22 @@ class ActionTypeHint(Action):
             return argcomplete_warn_redraw_prompt(prefix, msg)
 
 
+def raise_unexpected_value(message: str, val: Any = inspect._empty, exception: Optional[Exception] = None) -> NoReturn:
+    if val is not inspect._empty:
+        message += f'. Got value: {val}'
+    raise ValueError(message) from exception
+
+
+def raise_union_unexpected_value(uniontype, val: Any, exceptions: List[Exception]) -> NoReturn:
+    errors = indent_text('\n- '.join(str(e) for e in [''] + exceptions))
+    errors = errors.replace(f'. Got value: {val}', '').replace(f' {val} ', ' ')
+    subtypes = uniontype.__args__
+    raise ValueError(
+        f'Does not validate against any of the Union subtypes\nSubtypes: {subtypes}'
+        f'\nErrors:{errors}\nGiven value type: {type(val)}\nGiven value: {val}'
+    ) from exceptions[0]
+
+
 def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, prev_val=None, append=False, sub_add_kwargs=None):
 
     adapt_kwargs = {
@@ -514,19 +545,19 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
             subtypes = Union[tuple(set(type(v) for v in subtypehints))]
             val = adapt_typehints(val, subtypes, **adapt_kwargs)
         if val not in subtypehints:
-            raise ValueError(f'Expected a {typehint} but got "{val}"')
+            raise_unexpected_value(f'Expected a {typehint}', val)
 
     # Basic types
     elif typehint in leaf_types:
-        if not isinstance(val, typehint) and isinstance(val, str) and typehint is not str:
+        if isinstance(val, str) and typehint is not str:
             try:
                 val = yaml_load(val)
             except pyyaml_exceptions:
                 pass
-            if typehint is float and isinstance(val, int):
-                val = float(val)
-        if not isinstance(val, typehint):
-            raise ValueError(f'Expected a {typehint} but got "{val}"')
+        if typehint is float and isinstance(val, int) and not isinstance(val, bool):
+            val = float(val)
+        if not isinstance(val, typehint) or (typehint in (int, float) and isinstance(val, bool)):
+            raise_unexpected_value(f'Expected a {typehint}', val)
 
     # Registered types
     elif get_registered_type(typehint):
@@ -546,9 +577,12 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
                 val = val.name
             else:
                 if val not in typehint.__members__:
-                    raise ValueError(f'Value "{val}" is not a valid member name for the enum "{typehint}"')
+                    raise_unexpected_value(f'Expected a member of {typehint}', val)
         elif not serialize and not isinstance(val, typehint):
-            val = typehint[val]
+            try:
+                val = typehint[val]
+            except KeyError as ex:
+                raise_unexpected_value(f'Expected a member of {typehint}', val, ex)
 
     # Type
     elif typehint in {Type, type} or typehint_origin in {Type, type}:
@@ -559,7 +593,7 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
             val = import_object(val)
             if (typehint in {Type, type} and not isinstance(val, type)) or \
                (typehint not in {Type, type} and not is_subclass(val, subtypehints[0])):
-                raise ValueError(f'Value "{path}" is not a {typehint}.')
+                raise_unexpected_value(f'Expected an import path corresponding to a {typehint}', path)
 
     # Union
     elif typehint_origin == Union:
@@ -572,31 +606,31 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
             except Exception as ex:
                 vals.append(ex)
         if all(isinstance(v, Exception) for v in vals):
-            e = indent_text('\n- '.join(str(v) for v in ['']+vals))
-            raise ValueError(f'Value "{val}" does not validate against any of the types in {typehint}:{e}') from vals[0]
+            raise_union_unexpected_value(typehint, val, vals)
         val = [v for v in vals if not isinstance(v, Exception)][0]
 
     # Tuple or Set
     elif typehint_origin in tuple_set_origin_types:
         if not isinstance(val, (list, tuple, set)):
-            raise ValueError(f'Expected a {typehint_origin} but got "{val}"')
+            raise_unexpected_value(f'Expected a {typehint_origin}', val)
         val = list(val)
         if subtypehints is not None:
             is_tuple = typehint_origin in {Tuple, tuple}
             is_ellipsis = is_ellipsis_tuple(typehint)
             if is_tuple and not is_ellipsis and len(val) != len(subtypehints):
-                raise ValueError(f'Expected a tuple with {len(subtypehints)} elements but got "{val}"')
+                raise_unexpected_value(f'Expected a tuple with {len(subtypehints)} elements', val)
             for n, v in enumerate(val):
                 subtypehint = subtypehints[0 if is_ellipsis or not is_tuple else n]
                 val[n] = adapt_typehints(v, subtypehint, **adapt_kwargs)
             if is_tuple and len(val) == 0:
-                raise ValueError('Expected a non-empty tuple')
+                raise_unexpected_value(f'Expected a non-empty tuple', val)
         if not serialize:
             val = tuple(val) if typehint_origin in {Tuple, tuple} else set(val)
 
     # List, Iterable or Sequence
     elif typehint_origin in sequence_origin_types:
         if append:
+            adapt_kwargs.pop('prev_val')
             if prev_val is None:
                 prev_val = []
             elif not isinstance(prev_val, list):
@@ -604,12 +638,13 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
                     prev_val = [adapt_typehints(prev_val, subtypehints[0], **adapt_kwargs)]
                 except Exception:
                     prev_val = []
-            val = prev_val + (val if isinstance(val, list) else [adapt_typehints(val, subtypehints[0], **adapt_kwargs)])
-            prev_val = prev_val + [None] * (len(val) if isinstance(val, list) else 1)
+            val_is_list = isinstance(val, list)
+            val = prev_val + (val if val_is_list else [val])
+            prev_val = prev_val + [None] * (len(val)-len(prev_val) if val_is_list else 1)
         if isinstance(val, NestedArg) and subtypehints is not None:
             val = (prev_val[:-1] if isinstance(prev_val, list) else []) + [val]
         elif not isinstance(val, list):
-            raise ValueError(f'Expected a List but got "{val}"')
+            raise_unexpected_value(f'Expected a {typehint_origin}', val)
         if subtypehints is not None:
             for n, v in enumerate(val):
                 adapt_kwargs_n = {**adapt_kwargs, 'prev_val': prev_val[n]} if isinstance(prev_val, list) else adapt_kwargs
@@ -623,7 +658,7 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
             else:
                 val = {val.key: val.val}
         elif not isinstance(val, dict):
-            raise ValueError(f'Expected a Dict but got "{val}"')
+            raise_unexpected_value(f'Expected a {typehint_origin}', val)
         if subtypehints is not None:
             if subtypehints[0] == int:
                 cast = str if serialize else int
@@ -649,27 +684,31 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
                 val = object_path_serializer(val)
         else:
             try:
+                val_input = val
                 if isinstance(val, str):
-                    val_obj = import_object(val)
+                    class_path = val
+                    return_type = get_callable_return_type(typehint)
+                    if '.' not in val and return_type:
+                        class_path = resolve_class_path_by_name(return_type, val)
+                    val_obj = import_object(class_path)
                     if inspect.isclass(val_obj):
-                        val = Namespace(class_path=val)
+                        val = Namespace(class_path=class_path)
                     elif callable(val_obj):
                         val = val_obj
                     else:
                         raise ImportError(f'Unexpected import object {val_obj}')
                 if isinstance(val, (dict, Namespace, NestedArg)):
-                    val_input = val
                     val = subclass_spec_as_namespace(val, prev_val)
                     if not is_subclass_spec(val):
                         raise ImportError(f'Dict must include a class_path and optionally init_args, but got {val_input}')
-                    val, partial_classes, num_partial_args = get_partial_callable_classes(typehint, val)
+                    val, partial_classes, num_partial_args = adapt_partial_callable_class(typehint, val)
                     val_class = import_object(val['class_path'])
                     if inspect.isclass(val_class) and not (partial_classes or callable_instances(val_class)):
-                        raise ImportError(f'Expected {val["class_path"]!r} to be a class that instantiates into callable or a subclass of {partial_classes}.')
+                        raise ImportError(f'Expected {val["class_path"]!r} to be a class that instantiates into callable or a subclass of {partial_classes}')
                     val['class_path'] = get_import_path(val_class)
                     val = adapt_class_type(val, False, instantiate_classes, sub_add_kwargs, skip_args=num_partial_args)
             except (ImportError, AttributeError, ParserError) as ex:
-                raise ValueError(f'Type {typehint} expects a function or a callable class: {ex}') from ex
+                raise_unexpected_value(f'Type {typehint} expects a function or a callable class: {ex}', val, ex)
 
     # Subclass
     elif not hasattr(typehint, '__origin__') and inspect.isclass(typehint):
@@ -683,21 +722,24 @@ def adapt_typehints(val, typehint, serialize=False, instantiate_classes=False, p
         val_input = val
         val = subclass_spec_as_namespace(val, prev_val)
         if not is_subclass_spec(val):
-            raise ValueError(
-                f'Type {typehint} expects: a class path (str); or a dict with a class_path entry; '
-                f'or a dict with init_args (if class path given previously). Got "{val_input}".'
+            raise_unexpected_value(
+                f'Not a valid {typehint.__name__}. Got value: {val_input}\n'
+                'Subclass types expect one of:\n'
+                '- a class path (str)\n'
+                '- a dict with class_path entry\n'
+                '- a dict without class_path but with init_args entry (class path given previously)'
             )
 
         try:
             val_class = import_object(resolve_class_path_by_name(typehint, val['class_path']))
             if not is_subclass(val_class, typehint):
-                raise ValueError(f'"{val["class_path"]}" is not a subclass of {typehint}')
+                raise_unexpected_value(f'Import path {val["class_path"]} does not correspond to a subclass of {typehint}')
             val['class_path'] = get_import_path(val_class)
             val = adapt_class_type(val, serialize, instantiate_classes, sub_add_kwargs, prev_val=prev_val)
         except (ImportError, AttributeError, AssertionError, ParserError) as ex:
             class_path = val if isinstance(val, str) else val['class_path']
-            e = indent_text(f'\n- {ex}')
-            raise ValueError(f'Problem with given class_path "{class_path}":{e}') from ex
+            error = indent_text(f'\n- {ex}')
+            raise_unexpected_value(f'Problem with given class_path {class_path!r}:{error}', exception=ex)
 
     return val
 
@@ -746,23 +788,37 @@ def subclass_spec_as_namespace(val, prev_val=None):
     return val
 
 
-def get_partial_callable_classes(callable_type, subclass_spec):
+def get_callable_return_type(typehint):
+    return_type = None
+    if len(getattr(typehint, '__args__', None) or []) > 1:
+        return_type = typehint.__args__[1]
+    return return_type
+
+
+def get_subclass_types(typehint, callable_return=True):
+    subclass_types = None
+    if callable_return and ActionTypeHint.is_callable_typehint(typehint, all_subtypes=False) and typehint.__args__:
+        typehint = typehint.__args__[-1]
+    if ActionTypeHint.is_subclass_typehint(typehint, all_subtypes=False):
+        if get_typehint_origin(typehint) == Union:
+            subclass_types = tuple(t for t in typehint.__args__ if ActionTypeHint.is_subclass_typehint(t))
+        else:
+            subclass_types = (typehint,)
+    return subclass_types
+
+
+def adapt_partial_callable_class(callable_type, subclass_spec):
     partial_classes = False
     num_partial_args = 0
-    subtypes = getattr(callable_type, '__args__', None)
-    if subtypes and len(subtypes) > 1:
-        return_type = subtypes[-1]
-        return_origin = get_typehint_origin(return_type)
-        subclass_types = tuple(
-            t for t in (return_type.__args__ if return_origin == Union else [return_type])
-            if ActionTypeHint.is_subclass_typehint(t)
-        )
-        class_type = import_object(resolve_class_path_by_name(return_type, subclass_spec['class_path']))
+    return_type = get_callable_return_type(callable_type)
+    if return_type:
+        subclass_types = get_subclass_types(return_type, callable_return=False)
+        class_type = import_object(resolve_class_path_by_name(return_type, subclass_spec.class_path))
         if subclass_types and is_subclass(class_type, subclass_types):
-            subclass_spec = deepcopy(subclass_spec)
+            subclass_spec = subclass_spec.clone()
             subclass_spec['class_path'] = get_import_path(class_type)
             partial_classes = True
-            num_partial_args = len(subtypes)-1
+            num_partial_args = len(callable_type.__args__)-1
     return subclass_spec, partial_classes, num_partial_args
 
 
@@ -964,12 +1020,6 @@ def sort_subtypes_for_append(subtypes):
     return subtypes
 
 
-def get_typehint_origin(typehint):
-    if not hasattr(typehint, '__origin__') and get_import_path(typehint.__class__) == 'types.UnionType':
-        return Union
-    return getattr(typehint, '__origin__', None)
-
-
 def is_ellipsis_tuple(typehint):
     return typehint.__origin__ in {Tuple, tuple} and len(typehint.__args__) > 1 and typehint.__args__[1] == Ellipsis
 
