@@ -14,7 +14,7 @@ from functools import wraps
 from types import BuiltinFunctionType, FunctionType, ModuleType
 from typing import Any, Callable, Optional, Tuple, Type, TypeVar, Union
 
-from .loaders_dumpers import load_value
+from .loaders_dumpers import json_dump, load_value
 from .optionals import (
     fsspec_support,
     get_config_read_mode,
@@ -259,6 +259,12 @@ def object_path_serializer(value):
         raise ValueError(f'Only possible to serialize an importable object, given {value}: {ex}') from ex
 
 
+def get_typehint_origin(typehint):
+    if not hasattr(typehint, '__origin__') and get_import_path(typehint.__class__) == 'types.UnionType':
+        return Union
+    return getattr(typehint, '__origin__', None)
+
+
 lenient_check: ContextVar = ContextVar('lenient_check', default=False)
 
 
@@ -285,8 +291,30 @@ def change_to_path_dir(path: Optional['Path']):
             os.chdir(cwd)
 
 
+def hash_item(item):
+    try:
+        if isinstance(item, (dict, list)):
+            item_hash = hash(json_dump(item))
+        else:
+            item_hash = hash(item)
+    except Exception:
+        item_hash = hash(repr(item))
+    return item_hash
+
+
+def unique(iterable):
+    unique_items = []
+    seen = set()
+    for item in iterable:
+        key = hash_item(item)
+        if key not in seen:
+            unique_items.append(item)
+            seen.add(key)
+    return unique_items
+
+
 def iter_to_set_str(val, sep=','):
-    val = list(val)
+    val = unique(val)
     if len(val) == 1:
         return str(val[0])
     return '{'+sep.join(str(x) for x in val)+'}'
@@ -558,12 +586,12 @@ class LoggerProperty:
 
     def __init__(self, *args, logger: Union[bool, str, dict, logging.Logger] = False, **kwargs):
         """Initializer for LoggerProperty class."""
-        self.logger = logger
+        self.logger = logger  # type: ignore
         super().__init__(*args, **kwargs)
 
 
     @property
-    def logger(self):
+    def logger(self) -> logging.Logger:
         """The logger property for the class.
 
         :getter: Returns the current logger.
