@@ -1,15 +1,21 @@
 #!/usr/bin/env python3
 
 import calendar
+import inspect
 import logging
 import unittest
+import xml.dom
+from calendar import Calendar
 from contextlib import contextmanager
 from random import shuffle
-from typing import Any, Dict
+from typing import Any, Callable, Dict, List
 from unittest.mock import patch
 from jsonargparse import class_from_function, Namespace
 from jsonargparse.optionals import docstring_parser_support
-from jsonargparse.parameter_resolvers import get_signature_parameters as get_params
+from jsonargparse.parameter_resolvers import (
+    get_signature_parameters as get_params,
+    is_lambda,
+)
 
 
 class ClassA:
@@ -148,19 +154,23 @@ class ClassG:
         elif self.func == '2':
             self.method2(**self.kws)
 
-    def method1(self, kmg1: int, kmg2: str, kmg3: bool):
+    def method1(self, kmg1: int = 1, kmg2: str = '-', kmg3: bool = True):
         """
         Args:
             kmg1: help for kmg1
+            kmg2: help for kmg2
             kmg3: help for kmg3
         """
+        self.called = 'method1'
 
-    def method2(self, kmg1: int, kmg2: float, kmg3: bool, kmg4: int):
+    def method2(self, kmg1: int = 1, kmg2: float = 2.3, kmg3: bool = False, kmg4: int = 4):
         """
         Args:
             kmg1: help for kmg1
             kmg3: help for kmg3
+            kmg4: help for kmg4
         """
+        self.called = 'method2'
 
 class ClassM1:
     def __init__(self, km1: int = 1):
@@ -258,6 +268,44 @@ class ClassU5:
     def method1(self, kws: dict):
         pass
 
+class Optimizer:
+    def __init__(self, params: List[int]):
+        self.params = params
+
+class SGD(Optimizer):
+    def __init__(self, params: List[int], lr: float, **kwargs):
+        super().__init__(params, **kwargs)
+        self.lr = lr
+
+class ClassInstanceDefaults:
+    def __init__(
+        self,
+        layers: int,
+        number: float = 0.2,
+        elements: Callable[..., List[int]] = lambda *a: list(a),
+        node: Callable[[], xml.dom.Node] = lambda: xml.dom.Node(),
+        cal1: Calendar = Calendar(firstweekday=1),
+        cal2: Calendar = calendar.TextCalendar(2),
+        opt1: Callable[[List[int]], Optimizer] = lambda p: SGD(p, lr=0.01),
+        opt2: Callable[[List[int]], Optimizer] = lambda p: SGD(p, 0.02),
+        opt3: Callable[[List[int]], Optimizer] = lambda p, lr=0.1: SGD(p, lr=lr),  # type: ignore
+        opt4: Callable[[List[int]], Optimizer] = lambda p: Calendar(firstweekday=3),  # type: ignore
+        **kwargs,
+    ):
+        """
+        Args:
+            layers: help for layers
+            number: help for number
+            elements: help for elements
+            node: help for node
+            cal1: help for cal1
+            cal2: help for cal2
+            opt1: help for opt1
+            opt2: help for opt2
+            opt3: help for opt3
+            opt4: help for opt4
+        """
+
 def function_no_args_no_kwargs(pk1: str, k2: int = 1):
     """
     Args:
@@ -290,6 +338,7 @@ def function_make_class_b(*args, k1: str = '-', **kwargs):
 def function_pop_get_from_kwargs(kn1: int = 0, **kw):
     """
     Args:
+        k2: help for k2
         kn1: help for kn1
         kn2: help for kn2
         kn3: help for kn3
@@ -299,7 +348,9 @@ def function_pop_get_from_kwargs(kn1: int = 0, **kw):
     kw.pop('kn2', 0.5)
     kw.get('kn3', {})
     kw.pop('kn4', [1])
-    return function_no_args_no_kwargs(**kw)
+    kw.get('pk1', '')
+    function_no_args_no_kwargs(**kw)
+    kw.pop('pk1', '')
 
 def function_with_bug(**kws):
     return does_not_exist(**kws)  # pylint: disable=undefined-variable
@@ -313,16 +364,60 @@ def function_module_class(**kwds):
     return calendar.Calendar(**kwds)
 
 
+constant_boolean_1 = True
+constant_boolean_2 = False
+
+
+def function_constant_boolean(**kwargs):
+    if constant_boolean_1:
+        return function_with_kwargs(**kwargs)
+    elif not constant_boolean_2:
+        return function_with_kwargs(k1=False, **kwargs)
+
+def cond_1(kc: int = 1, kn0: str = 'x', kn1: str = '-'):
+    """
+    Args:
+        kc: help for kc
+        kn1: help for kn1
+    """
+
+def cond_2(kc: int = 1, kn2: bool = True):
+    """
+    Args:
+        kn2: help for kn2
+    """
+
+def cond_3(kc: int = 1, kn3: int = 2, kn4: float = 0.1):
+    """
+    Args:
+        kn3: help for kn3
+        kn4: help for kn4
+    """
+
+def conditional_calls(**kwargs):
+    if 'kn1' in kwargs:
+        cond_1(kn0='y', **kwargs)
+    elif 'kn2' in kwargs:
+        cond_2(**kwargs)
+    else:
+        cond_3(**kwargs)
+
+
 @contextmanager
 def source_unavailable():
     with patch('inspect.getsource', side_effect=OSError('could not get source code')):
         yield
 
 
-def assert_params(self, params, expected):
+def assert_params(self, params, expected, origins={}):
     self.assertEqual(expected, [p.name for p in params])
     docs = [f'help for {p.name}' for p in params] if docstring_parser_support else [None] * len(params)
     self.assertEqual(docs, [p.doc for p in params])
+    param_origins = {
+        n: [o.split(f'{__name__}.', 1)[1] for o in p.origin]
+        for n, p in enumerate(params) if p.origin is not None
+    }
+    self.assertEqual(param_origins, origins)
 
 
 logger = logging.getLogger('test_parameter_resolvers')
@@ -367,7 +462,16 @@ class GetClassParametersTests(unittest.TestCase):
             assert_params(self, get_params(ClassF1), [])
 
     def test_get_params_class_kwargs_in_attr_method_conditioned_on_arg(self):
-        assert_params(self, get_params(ClassG), ['func', 'kmg1', 'kmg3'])
+        assert_params(
+            self,
+            get_params(ClassG),
+            ['func', 'kmg1', 'kmg2', 'kmg3', 'kmg4'],
+            {
+                2: ['ClassG._run:3', 'ClassG._run:5'],
+                3: ['ClassG._run:3', 'ClassG._run:5'],
+                4: ['ClassG._run:5'],
+            },
+        )
         with source_unavailable():
             assert_params(self, get_params(ClassG), ['func'])
 
@@ -394,6 +498,24 @@ class GetClassParametersTests(unittest.TestCase):
             params = get_params(class_a)
             assert_params(self, params, ['pk1', 'k2'])
 
+    def test_get_params_class_instance_defaults(self):
+        params = get_params(ClassInstanceDefaults)
+        assert_params(self, params, ['layers', 'number', 'elements', 'node', 'cal1', 'cal2', 'opt1', 'opt2', 'opt3', 'opt4'])
+        with self.subTest('unmodified defaults'):
+            self.assertEqual(params[0].default, inspect._empty)
+            self.assertEqual(params[1].default, 0.2)
+            self.assertTrue(is_lambda(params[2].default))
+        with self.subTest('supported defaults'):
+            self.assertEqual(params[3].default, dict(class_path='xml.dom.Node'))
+            self.assertEqual(params[4].default, dict(class_path='calendar.Calendar', init_args=dict(firstweekday=1)))
+            self.assertEqual(params[6].default, dict(class_path=f'{__name__}.SGD', init_args=dict(lr=0.01)))
+        with self.subTest('unsupported defaults'):
+            self.assertIsInstance(params[5].default, calendar.TextCalendar)
+            self.assertTrue(is_lambda(params[7].default))
+            self.assertTrue(is_lambda(params[9].default))
+        with self.subTest('invalid defaults'):
+            self.assertTrue(is_lambda(params[8].default))
+
 
 class GetMethodParametersTests(unittest.TestCase):
 
@@ -455,8 +577,7 @@ class GetFunctionParametersTests(unittest.TestCase):
     def test_get_params_function_pop_get_from_kwargs(self):
         with self.assertLogs(logger, level='DEBUG') as log:
             params = get_params(function_pop_get_from_kwargs, logger=logger)
-            assert_params(self, params, ['kn1', 'kn2', 'kn3', 'kn4', 'pk1', 'k2'])
-            self.assertIs(params[-1].annotation, int)
+            assert_params(self, params, ['kn1', 'k2', 'kn2', 'kn3', 'kn4', 'pk1'])
             self.assertIn('unsupported kwargs pop/get default', log.output[0])
         with source_unavailable():
             assert_params(self, get_params(function_pop_get_from_kwargs), ['kn1'])
@@ -465,6 +586,28 @@ class GetFunctionParametersTests(unittest.TestCase):
         params = get_params(function_module_class)
         self.assertEqual(['firstweekday'], [p.name for p in params])
 
+    def test_get_params_function_constant_boolean(self):
+        assert_params(self, get_params(function_constant_boolean), ['k1', 'pk1', 'k2'])
+        with patch.dict(function_constant_boolean.__globals__, {'constant_boolean_1': False}):
+            assert_params(self, get_params(function_constant_boolean), ['pk1', 'k2'])
+            with patch.dict(function_constant_boolean.__globals__, {'constant_boolean_2': True}):
+                self.assertEqual(get_params(function_constant_boolean), [])
+
+    def test_conditional_calls_kwargs(self):
+        assert_params(
+            self,
+            get_params(conditional_calls),
+            ['kc', 'kn1', 'kn2', 'kn3', 'kn4'],
+            {
+                1: ["conditional_calls:3"],
+                2: ['conditional_calls:5'],
+                3: ['conditional_calls:7'],
+                4: ['conditional_calls:7'],
+            },
+        )
+        with source_unavailable():
+            self.assertEqual(get_params(conditional_calls), [])
+
 
 class OtherTests(unittest.TestCase):
 
@@ -502,7 +645,6 @@ class OtherTests(unittest.TestCase):
         with self.assertLogs(logger, level='DEBUG') as log:
             self.assertEqual([], get_params(function_with_bug, logger=logger))
             self.assertIn('does_not_exist', log.output[0])
-            self.assertIn('Problems with AST resolving', log.output[0])
 
     def test_get_params_failures(self):
         self.assertRaises(ValueError, lambda: get_params('invalid'))
