@@ -73,7 +73,7 @@ class SignaturesTests(unittest.TestCase):
                          c3_a5: Union[int, float, str, List[int], Dict[str, float]] = 5,
                          c3_a6: Optional[Class1] = None,
                          c3_a7: Tuple[str, int, float] = ('7', 7, 7.0),
-                         c3_a8: Tuple[str, Class1] = None,
+                         c3_a8: Optional[Tuple[str, Class1]] = None,
                          c1_a5: str = 'five',
                          **kwargs):
                 """Class3 short description
@@ -209,6 +209,73 @@ class SignaturesTests(unittest.TestCase):
         self.assertEqual(cfg.a, Namespace(a1=4, a2=2.3))
 
 
+    def test_add_class_required_args(self):
+        class Model:
+            def __init__(self, n: int, m: float):
+                pass
+
+        parser = ArgumentParser(error_handler=None)
+        parser.add_class_arguments(Model, 'model')
+        parser.add_argument('--config', action=ActionConfigFile)
+
+        for args in [
+            [],
+            ['--model.n=2'],
+            ['--model.m=0.1'],
+            ['--model.n=x', '--model.m=0.1'],
+        ]:
+            with self.assertRaises(ParserError):
+                parser.parse_args(args)
+
+        out = StringIO()
+        with redirect_stdout(out), self.assertRaises(SystemExit):
+            parser.parse_args(['--model.m=0.1', '--print_config'])
+        self.assertIn('  n: null', out.getvalue())
+
+        cfg = parser.parse_args([f'--config={out.getvalue()}', '--model.n=3'])
+        self.assertEqual(cfg.model, Namespace(m=0.1, n=3))
+
+
+    def test_add_class_conditional_kwargs(self):
+        from jsonargparse_tests.test_parameter_resolvers import ClassG
+
+        parser = ArgumentParser()
+        parser.add_class_arguments(ClassG, 'g')
+
+        cfg = parser.get_defaults()
+        self.assertEqual(cfg.g, Namespace(func=None, kmg1=1))
+
+        cfg = parser.parse_args(['--g.func=1', '--g.kmg2=x'])
+        self.assertEqual(cfg.g, Namespace(func='1', kmg1=1, kmg2='x'))
+        init = parser.instantiate_classes(cfg)
+        init.g._run()
+        self.assertEqual(init.g.called, 'method1')
+
+        cfg = parser.parse_args(['--g.func=2', '--g.kmg4=5'])
+        self.assertEqual(cfg.g, Namespace(func='2', kmg1=1, kmg4=5))
+        init = parser.instantiate_classes(cfg)
+        init.g._run()
+        self.assertEqual(init.g.called, 'method2')
+
+        help_str = StringIO()
+        parser.print_help(help_str)
+        module = 'jsonargparse_tests.test_parameter_resolvers'
+        expected = [
+            f'origins: {module}.ClassG._run:3; {module}.ClassG._run:5',
+            f'origins: {module}.ClassG._run:5',
+        ]
+        if docstring_parser_support:
+            expected += [
+                'help for func (required, type: str)',
+                'help for kmg1 (type: int, default: 1)',
+                'help for kmg2 (type: Union[str, float], default: {-, 2.3})',
+                'help for kmg3 (type: bool, default: {True, False})',
+                'help for kmg4 (type: int, default: 4)',
+            ]
+        for value in expected:
+            self.assertIn(value, help_str.getvalue())
+
+
     def test_add_method_arguments(self):
 
         class MyClass:
@@ -438,6 +505,38 @@ class SignaturesTests(unittest.TestCase):
             self.assertIn("discarding init_args: {'pa': 'A', 'pc': 'X'}", str(w[0].message))
 
 
+    def test_add_subclass_nested_discard_init_args(self):
+        class ChildBase:
+            pass
+
+        class A(ChildBase):
+            def __init__(self, a: int = 0):
+                pass
+
+        class B(ChildBase):
+            def __init__(self, b: int = 0):
+                pass
+
+        class Parent:
+            def __init__(self, c: ChildBase):
+                pass
+
+        with mock_module(Parent, ChildBase, A, B) as module, warnings.catch_warnings(record=True) as w:
+            parser = ArgumentParser(error_handler=None)
+            parser.add_subclass_arguments(Parent, 'p')
+            cfg = parser.parse_args([
+                '--p=Parent',
+                '--p.init_args.c=A',
+                '--p.init_args.c.init_args.a=1',
+                '--p.init_args.c=B',
+                '--p.init_args.c.init_args.b=2',
+            ])
+            self.assertEqual(cfg.p.class_path, f'{module}.Parent')
+            self.assertEqual(cfg.p.init_args.c.class_path, f'{module}.B')
+            self.assertEqual(cfg.p.init_args.c.init_args, Namespace(b=2))
+            self.assertIn("discarding init_args: {'a': 1}", str(w[0].message))
+
+
     def test_class_path_override_with_mixed_type(self):
         class MyCalendar(Calendar):
             def __init__(self, *args, param: int = 0, **kwargs):
@@ -777,8 +876,8 @@ class SignaturesTests(unittest.TestCase):
             with self.assertLogs(logger=parser.logger, level='DEBUG') as log:
                 parser.add_class_arguments(Class2, skip={'c2_a2'})
                 self.assertEqual(1, len(log.output))
-                self.assertIn('"c2_a2" from "Class2.__init__"', log.output[0])
-                self.assertIn('Parameter requested to be skipped', log.output[0])
+                self.assertIn('parameter "c2_a2" from "', log.output[0])
+                self.assertIn('Class2.__init__" because of: Parameter requested to be skipped', log.output[0])
 
 
     def test_instantiate_classes(self):
@@ -843,7 +942,7 @@ class SignaturesTests(unittest.TestCase):
 
     def test_implicit_optional(self):
 
-        def func(a1: int = None):
+        def func(a1: Optional[int] = None):
             return a1
 
         parser = ArgumentParser(error_handler=None)
