@@ -30,6 +30,7 @@ from jsonargparse.typing import (
     Path_fc,
     Path_fr,
     path_type,
+    PositiveFloat,
     PositiveInt,
     register_type,
     restricted_number_type,
@@ -123,6 +124,22 @@ class TypeHintsTests(unittest.TestCase):
                     self.assertEqual(value, parser.parse_args([f'--cls.val={value}']).cls.init_args.val)
 
 
+    def test_boolean_not_a_number(self):
+        for argtype in [int, float, PositiveInt, PositiveFloat]:
+            with self.subTest(argtype):
+                parser = ArgumentParser(error_handler=None)
+                parser.add_argument('--num', type=argtype)
+                for value in [True, False]:
+                    with self.assertRaises(ParserError):
+                        parser.parse_object({'num': value})
+
+
+    def test_str_with_yaml_constructor_error(self):
+        parser = ArgumentParser()
+        parser.add_argument('--val', type=str)
+        self.assertEqual('{{something}}', parser.parse_args(['--val={{something}}']).val)
+
+
     def test_list(self):
         for list_type in [Iterable, List, Sequence]:
             with self.subTest(str(list_type)):
@@ -288,12 +305,13 @@ class TypeHintsTests(unittest.TestCase):
 
     def test_list_append(self):
         parser = ArgumentParser(error_handler=None)
-        parser.add_argument('--val', type=Union[int, str, List[int]])
+        parser.add_argument('--val', type=Union[int, float, List[int]])
         self.assertEqual(0, parser.parse_args(['--val=0']).val)
         self.assertEqual([0], parser.parse_args(['--val+=0']).val)
         self.assertEqual([1, 2, 3], parser.parse_args(['--val=1', '--val+=2', '--val+=3']).val)
         self.assertEqual([1, 2, 3], parser.parse_args(['--val=[1,2]', '--val+=3']).val)
-        self.assertEqual([1], parser.parse_args(['--val=a', '--val+=1']).val)
+        self.assertEqual([1], parser.parse_args(['--val=0.1', '--val+=1']).val)
+        self.assertRaises(ParserError, lambda: parser.parse_args(['--val=a', '--val+=1']))
         with warnings.catch_warnings(record=True) as w:
             self.assertEqual(3, parser.parse_args(['--val=[1,2]', '--val=3']).val)
             self.assertIn('Replacing list value "[1, 2]" with "3"', str(w[0].message))
@@ -307,6 +325,7 @@ class TypeHintsTests(unittest.TestCase):
         self.assertEqual([1, 2, 3], parser.parse_args(['--cfg', 'val+: 3']).val)
         self.assertEqual([1, 2, 3, 4], parser.parse_args(['--cfg', 'val+: [3, 4]']).val)
         self.assertRaises(ParserError, lambda: parser.parse_args(['--cfg', 'val+: a']))
+        self.assertRaises(ParserError, lambda: parser.parse_args(['--val=2', '--cfg', 'val+: 3']))
 
 
     def test_list_append_subclass_init_args(self):
@@ -343,9 +362,31 @@ class TypeHintsTests(unittest.TestCase):
         self.assertEqual(['TextCalendar', 'HTMLCalendar'], [c.class_path.split('.')[1] for c in cfg.cls.cal])
 
 
+    def test_list_append_subclass(self):
+        class A:
+            def __init__(self, cals: Optional[Union[Calendar, List[Calendar]]] = None):
+                self.cals = cals
+
+        parser = ArgumentParser(error_handler=None)
+        parser.add_class_arguments(A, 'a')
+        cfg = parser.parse_args([
+            '--a.cals+=Calendar',
+            '--a.cals.firstweekday=3',
+            '--a.cals+=TextCalendar',
+            '--a.cals.firstweekday=1',
+        ])
+        self.assertEqual(['calendar.Calendar', 'calendar.TextCalendar'], [x.class_path for x in cfg.a.cals])
+        self.assertEqual([3, 1], [x.init_args.firstweekday for x in cfg.a.cals])
+        cfg = parser.parse_args([f'--a={json.dumps(cfg.a.as_dict())}', '--a.cals.firstweekday=4'])
+        self.assertEqual(Namespace(firstweekday=4), cfg.a.cals[-1].init_args)
+        args = ['--a.cals+=Invalid', '--a.cals+=TextCalendar']
+        self.assertRaises(ParserError, lambda: parser.parse_args(args))
+        self.assertRaises(ParserError, lambda: parser.parse_args(args + ['--print_config']))
+
+
     def test_list_append_subcommand_subclass(self):
         class A:
-            def __init__(self, cals: Union[Calendar, List[Calendar]] = None):
+            def __init__(self, cals: Optional[Union[Calendar, List[Calendar]]] = None):
                 self.cals = cals
 
         parser = ArgumentParser(error_handler=None)
@@ -364,6 +405,9 @@ class TypeHintsTests(unittest.TestCase):
         self.assertEqual([3, 1], [x.init_args.firstweekday for x in cfg.cmd.a.cals])
         cfg = parser.parse_args(['cmd', f'--a={json.dumps(cfg.cmd.a.as_dict())}', '--a.cals.firstweekday=4'])
         self.assertEqual(Namespace(firstweekday=4), cfg.cmd.a.cals[-1].init_args)
+        args = ['cmd', '--a.cals+=Invalid', '--a.cals+=TextCalendar']
+        self.assertRaises(ParserError, lambda: parser.parse_args(args))
+        self.assertRaises(ParserError, lambda: parser.parse_args(args + ['--print_config']))
 
 
     def test_restricted_number_type(self):
@@ -594,10 +638,20 @@ class TypeHintsTests(unittest.TestCase):
     @unittest.skipIf(sys.version_info[:2] < (3, 10), 'new union syntax introduced in python 3.10')
     def test_union_new_syntax(self):
         parser = ArgumentParser(error_handler=None)
-        parser.add_argument('--str', type=eval('int | None'))
-        self.assertEqual(123, parser.parse_args(['--str=123']).str)
-        self.assertIsNone(parser.parse_args(['--str=null']).str)
-        self.assertRaises(ParserError, lambda: parser.parse_args(['--str=abc']))
+        parser.add_argument('--val', type=eval('int | None'))
+        self.assertEqual(123, parser.parse_args(['--val=123']).val)
+        self.assertIsNone(parser.parse_args(['--val=null']).val)
+        self.assertRaises(ParserError, lambda: parser.parse_args(['--val=abc']))
+
+
+    @unittest.skipIf(sys.version_info[:2] < (3, 10), 'new union syntax introduced in python 3.10')
+    def test_union_new_syntax_subclasses(self):
+        parser = ArgumentParser(error_handler=None)
+        parser.add_argument('--op', type=eval('Calendar | bool'))
+        out = StringIO()
+        with redirect_stdout(out), self.assertRaises(SystemExit):
+            parser.parse_args(['--op.help=TextCalendar'])
+        self.assertIn('--op.init_args.firstweekday', out.getvalue())
 
 
     def test_Callable_with_function_path(self):
@@ -747,6 +801,8 @@ class TypeHintsTests(unittest.TestCase):
             self.assertEqual(optim.params, [4.5, 6.7])
             self.assertEqual(optim.lr, 0.01)
 
+            self.assertEqual(cfg, parser.parse_args(['--optimizer=Adam', '--optimizer.lr=0.01']))
+
             help_str = StringIO()
             parser.print_help(help_str)
             for name in ['Optimizer', 'SGD', 'Adam']:
@@ -1412,6 +1468,7 @@ class TypeHintsTmpdirTests(TempDirTestCase):
             self.assertEqual(cfg.nums, [0, 1, 2])
             cfg = parser.parse_args(['--nums+=[2, 3]'])
             self.assertEqual(cfg.nums, [0, 1, 2, 3])
+            self.assertEqual(str(cfg.__default_config__), str(config_path))
 
         with self.subTest('two default config appends'):
             config_path2 = pathlib.Path(self.tmpdir, 'config2.yaml')
@@ -1419,9 +1476,10 @@ class TypeHintsTmpdirTests(TempDirTestCase):
             parser.default_config_files += [str(config_path2)]
             cfg = parser.get_defaults()
             self.assertEqual(cfg.nums, [0, 1, 2])
+            self.assertEqual([str(c) for c in cfg.__default_config__], [str(config_path), str(config_path2)])
 
 
-    def test_list_append_default_config_files_subcommand(self):
+    def test_list_append_subcommand_global_default_config_files(self):
         config_path = pathlib.Path(self.tmpdir, 'config.yaml')
         parser = ArgumentParser(default_config_files=[str(config_path)])
         subcommands = parser.add_subcommands()
@@ -1431,6 +1489,24 @@ class TypeHintsTmpdirTests(TempDirTestCase):
         config_path.write_text('sub:\n  nums: [1]\n')
         cfg = parser.parse_args(['sub', '--nums+=2'])
         self.assertEqual(cfg.sub.nums, [1, 2])
+        self.assertEqual(str(cfg.__default_config__), str(config_path))
+        cfg = parser.parse_args(['sub', '--nums+=2'], defaults=False)
+        self.assertEqual(cfg.sub.nums, [2])
+
+
+    def test_list_append_subcommand_subparser_default_config_files(self):
+        config_path = pathlib.Path(self.tmpdir, 'config.yaml')
+        parser = ArgumentParser()
+        subcommands = parser.add_subcommands()
+        subparser = ArgumentParser(default_config_files=[str(config_path)])
+        subparser.add_argument('--nums', type=List[int], default=[0])
+        subcommands.add_subcommand('sub', subparser)
+        config_path.write_text('nums: [1]\n')
+        cfg = parser.parse_args(['sub', '--nums+=2'])
+        self.assertEqual(cfg.sub.nums, [1, 2])
+        self.assertEqual(str(cfg.sub.__default_config__), str(config_path))
+        cfg = parser.parse_args(['sub', '--nums+=2'], defaults=False)
+        self.assertEqual(cfg.sub.nums, [2])
 
 
     def test_class_type_with_default_config_files(self):
